;***    MRCSYSCL.ASM - MRCI System Client EXAMPLE
;
;       MRCI version 1.00.08  02-Apr-1993
;
;       NOTE: This example is implemented assuming the code segment is
;             writeable, to simplify the exposition.
;
;       This is an example "system client" of the Microsoft Real-time
;       Compression API.  A "system client" is a device driver or TSR
;       that loads and stays resident for the lifetime of the system.
;       DoubleSpace, Flash File System 2, and network drivers are examples
;       of system clients.
;
;       By contrast, an "application client" is a program like MSBackup,
;       which loads, uses MRCI, and then terminates.
;
;       This example demonstrates the proper technique for:
;
;       1) Calling the MRCI server to get its capabilities
;       2) Calling the MRCI server to perform Standard Compress
;       3) Calling the MRCI server to perform Decompress


include utility.inc
include mrci.inc


;***********************
;* Private Definitions *
;***********************

cbBUFFER    equ     8192                ; Our compression block size


;****************
;* CODE segment *
;****************

code    segment
    assume  cs:code


;*************
;* DATA AREA *
;*************

pmi     dd  0                           ; Pointer to MRCINFO structure

buffer1 db  cbBUFFER dup (?)            ; One data buffer
buffer2 db  cbBUFFER dup (?)            ; Another data buffer

mr      MRCREQUEST    <>                ; MRCI request block


;*********************
;* USE MRCI Services *
;*********************


main    proc
        call    InitMRC                 ; Get MRCI server information
        or      ax,ax                   ; MRCI server present
        jnz     max                     ;   NO, exit

;*  Compress some data
;
;       Assume buffer1 has uncompressed data
;
        mov     ax,cs                   ; Get CS in AX, for buffer pointers
        mov     ds,ax
        mov     si,OFFSET mr            ; ds:si -> our MRCREQUEST packet

        ;* Set source pointer, length
        mov     ds:[si].mr_pbSrc.offst,OFFSET buffer1
        mov     ds:[si].mr_pbSrc.segmt,ax
        mov     ds:[si].mr_cbSrc,cbBUFFER

        ;* Set destination pointer, length
        mov     ds:[si].mr_pbDst.offst,OFFSET buffer2
        mov     ds:[si].mr_pbDst.segmt,ax
        mov     ds:[si].mr_cbDst,cbBUFFER

        ;* Set storage chunk granularity
        mov     ds:[si].mr_cbChunk,1    ; We store at byte granularity

        ;* Get MRCINFO pointer for call
        les     bx,cs:pmi

        ;* Set operation and client type
        mov     ax,micapSTANDARD
        mov     cx,mcSYSTEM

        call    MRCCompress             ; Compress!
        or      ax,ax                   ; Did compress work?
        jnz     max                     ;   NO, go exit

        ;*  buffer2 now has compressed data, and
        ;   ds:[si].mr_cbDst has the size of the compressed data.


;*  Decompress some data
;
;       Assume buffer1 has compressed data
;
        mov     ax,cs                   ; Get CS in AX, for buffer pointers
        mov     ds,ax
        mov     si,OFFSET mr            ; ds:si -> our MRCREQUEST packet

        ;* Set source pointer, length
        mov     ds:[si].mr_pbSrc.offst,OFFSET buffer1
        mov     ds:[si].mr_pbSrc.segmt,ax
        mov     ds:[si].mr_cbSrc,cbBUFFER

        ;* Set destination pointer, length
        mov     ds:[si].mr_pbDst.offst,OFFSET buffer2
        mov     ds:[si].mr_pbDst.segmt,ax
        mov     ds:[si].mr_cbDst,cbBUFFER

        ;* Get MRCINFO pointer
        les     bx,cs:pmi

        ;* Set client type
        mov     cx,mcSYSTEM

        call    MRCDecompress           ; Compress!
        or      ax,ax                   ; Did compress work?
        jnz     max                     ;   NO, go exit

        ;*  buffer2 now has decompressed data, and
        ;   ds:[si].mr_cbDst has the size of the decompressed data.

;*  Do something else interesting

        ;* Nothing else to do, this is just an example.

max:    ret

main    endp


;** BEGIN: Include code for MRCCompress/MRCDecompress here!
;
        DefineMRCCompress
        DefineMRCDecompress
;
;** END:   Include code for MRCCompress/MRCDecompress here!


;******************
;* INIT-TIME CODE *
;******************

;** BEGIN: Include code for MRCQuery here!
;
        DefineMRCQuery
;
;** END:   Include code for MRCQuery here!


;**     InitMRC - Get address of MRCI Server
;
;
;       Entry
;           none.
;
;       Exit-Success
;           ax = 0
;           pmi = pointer to MRCI server MRCINFO structure
;
;       Exit-Failure
;           ax != 0, no MRCI server present
;
;       Uses
;           ax,bx,es,flags

InitMRC proc    near
    assume  ds:nothing,es:nothing,ss:nothing
        call    MRCQuery                ; es:bx -> MRCINFO, if server present
        or      ax,ax                   ; Is server present?
        jnz     imx                     ;   NO, go fail (ax != 0)

;*  Server present, save MRCINFO pointer

        mov     cs:pmi.offst,bx ; Store offset
        mov     ax,es
        mov     cs:pmi.segmt,ax ; Store segment

        xor     ax,ax                   ; Indicate success

imx:    ret

InitMRC endp


code    ends

        end
