;***    UTILITY.INC - Utility macros to make MASM code more readible
;
;       MRCI version 1.00.08  02-Apr-1993


;***    FARPTR - far pointer structure, to make code more readable
;
;       Instead of:
;           mov si,word ptr es:[bx]     ; si = offset
;           mov ds,word ptr es:[bx+2]   ; ds = segment
;
;       This structure permits:
;           mov si,es:[bx].offst
;           mov ds,es:[bx].segmt
;
FARPTR  struc
offst   dw      ?
segmt   dw      ?
FARPTR  ends


;***    WORD2 - Two-word structure, to make code more readable
;
;       Instead of:
;           mov ax,word ptr es:[bx]     ; ax = low word
;           mov dx,word ptr es:[bx+2]   ; dx = high word
;
;       This structure permits:
;           mov ax,es:[bx].loword
;           mov dx,es:[bx].highword
;
WORD2   struc
loword  dw      ?
hiword  dw      ?
WORD2   ends


;***    SaveReg - save a set of registers (see also RestoreReg)
;
;       Entry
;           SaveReg <reg1,reg2,...,regN>
;
;       Exit
;           reg1..regN pushed on stack, regN pushed last.
;           Use RestoreReg <regN,...reg2,reg1> to restore.
;
SaveReg macro   reglist         ;; push those registers
    irp reg,<reglist>
        push    reg
    endm
endm


;***    RestoreReg - Restore a set of registers saved by SaveReg
;
;       Entry
;           Stack has result of "SaveReg <reg1,reg2,...,regN>" macro.
;           RestoreReg <regN,...,reg2,reg1>
;
;       Exit
;           reg1..regN restored from stack.
;
RestoreReg  macro   reglist     ;; pop those registers
    irp reg,<reglist>
        pop reg
    endm
endm


;***    MS-DOS Device Driver Request Packet Header
;
;
DDREQPACKET    struc
rpLength       db    ?              ; Length in bytes of request block
rpUnit         db    ?              ; Device unit number (block only)
rpFunction     db    ?              ; Request function number
rpStatus       dw    ?              ; Status of operation
               db    8 dup(?)       ; RESERVED (for queue links)
DDREQPACKET    ends

;***    STxxx - DDREQPACKET.rpStatus word masks
;
;
STERR          equ   8000h          ; Bit 15 - ERROR
STBUI          equ   0200h          ; Bit  9 - BUSY
STDONE         equ   0100h          ; Bit  8 - DONE
STECODE        equ   00FFh          ; Error code

;***    DEVxxx - DDREQPACKET.rpFunction codes
;
;
DEVINIT        equ   0              ;INITIALIZATION
